#ifndef MODULE_META_DATE_CHANGED_H
#define MODULE_META_DATE_CHANGED_H
/**
 * @brief 简要说明
 * @author 戴月腾
 * @date 2018-10-18
 */

#include"module_const.h"



enum EDIT_TYPE_ID
{
    ET_INSERT = 0, // 新增
    ET_UPDATE,	   // 修改
    ET_DELETE      // 删除
};

class module_meta_date_changed:public singleton_base<module_meta_date_changed>
{
private:
    friend class singleton_base<module_meta_date_changed>;
    module_meta_date_changed()
    {
    }

    ///已经实现的：'card', 'staff', 'vehicle', 'vehicle_extend', 'staff_extend', 'area', 'reader',
    /// map antenna  path
    ///提供了接口未实现的： setting  light  lights_group
    ///下面的是暂时不要处理的
    ///'drivingface', 'drivingface_render', 'dat_drivingface_warning_point',
    ///'rules', 'drivingface_vehicle', 'dat_handup_vehicle', 'lights_binding'
public:
    ///基础数据
    void accept(sio::message::ptr const& data)
    {
        std::string name="";
        if(!tool_map::try_get_value(name, JSON_KEY_NAME, data))
        {
            log_error("web发来的数据: 基础数据找不到name字段");
            return;
        }

        std_debug("基础数据 receive meta_data_changed: %s", name.c_str());

        int id=-1;
        tool_map::try_get_value(id, JSON_KEY_ID, data);

        std::string op_type="";
        tool_map::try_get_value(op_type, JSON_KEY_OP_TYPE, data);

        if(-1 != id && !op_type.empty())
        {
            EDIT_TYPE_ID edit_type_id;
            if(!try_get_edit_type_id(op_type, edit_type_id))
            {
                log_error("web发来的数据: 基础数据op_type字段错误：op_type=%s", op_type.c_str());
                return;
            }

            if(JSON_KEY_NAME_VEHICLE == name || JSON_KEY_NAME_VEHICLE_EXTEND == name)
            {
                deal_call_edit_vehicle(id, edit_type_id);
            }
            else if(JSON_KEY_NAME_STAFF == name || JSON_KEY_NAME_STAFF_EXTEND == name)
            {
                deal_call_edit_staff(id, edit_type_id);
            }
            else if(JSON_KEY_NAME_CARD == name)
            {
                deal_call_edit_card(id, edit_type_id);
            }
            else if(JSON_KEY_NAME_AREA == name)
            {
                deal_call_edit_area(id, edit_type_id);
            }
            else if(JSON_KEY_NAME_READER == name)
            {
                deal_call_edit_reader(id, edit_type_id);
            }
            else if ("antenna" == name)
            {
                deal_call_edit_antenna(id,edit_type_id);
            }
            else if(JSON_KEY_NAME_PATH == name)
            {
                deal_call_edit_path(id, edit_type_id);
            }
            else if(JSON_KEY_NAME_MAP == name)
            {
                deal_call_edit_map(id, edit_type_id);
            }
            else if (JSON_KEY_NAME_LIGHT == name)
            {
                deal_call_edit_light(id,edit_type_id);///待实现
            }
            else if ("lights_group" == name)
            {
                deal_call_edit_lights_group(id,edit_type_id);///待实现
            }
            else
            {
                log_error("web发来的数据: 基础数据name字段错误：name=%s", name.c_str());
            }
        }
        else
        {
            if(JSON_KEY_NAME_SETTING == name)
            {
                //阈值限制接口
                init_setting(); ///待实现
            }
            else
            {
                log_error("web发来的数据: 基础数据name字段错误：name=%s", name.c_str());
            }
        }
    }

    ///3.清除卡相关信息，区域相关，分站相关，考勤相关，清除定时器，
    ///速度，状态 呼救 呼叫 告警相关。即保留基础信息，其他的重置。
    static void clear_card(std::shared_ptr<card_location_base> card_ptr)
    {
        //删除
        card_ptr->del_card_pos();
        // 升井 删除所有报警信息
        for(int i=0; i < CARD_EVENT_COUNT_MAX; i++ )
        {
            if (i == EVENT_TYPE::ET_CARD_LOW_POWER_SERIOUS)
            {
                continue;
            }

            auto ev_ptr = event_list::instance()->get_event_card(
                        card_ptr->m_id, card_ptr->m_type, static_cast<EVENT_TYPE>(i));
            if(ev_ptr && !ev_ptr->is_end()) //
            {
                event_list::copy_event(card_ptr, ev_ptr);

                ev_ptr->m_status = ES_END;

                event_list::save_event(ev_ptr);
            }
        }

        card_ptr->clear();
    }

private:
    void deal_call_edit_vehicle(int id, EDIT_TYPE_ID edit_type_id);

    void deal_call_edit_staff(int id, EDIT_TYPE_ID edit_type_id);

    ///id64格式为10000006666
    void deal_call_edit_card(int id64, EDIT_TYPE_ID edit_type_id);

    void deal_call_edit_reader(int id, EDIT_TYPE_ID edit_type_id);

    ///待实现
    void deal_call_edit_path(int id, EDIT_TYPE_ID edit_type_id);

    void deal_call_edit_antenna(int id,EDIT_TYPE_ID edit_type_id);

    void deal_call_edit_area(int id, EDIT_TYPE_ID edit_type_id);

    void deal_call_edit_map(int id, EDIT_TYPE_ID edit_type_id);

    ///待实现
    void deal_call_edit_light(int id,EDIT_TYPE_ID edit_type_id);

    ///待实现
    void deal_call_edit_lights_group(int id,EDIT_TYPE_ID edit_type_id);

    ///待实现
    void init_setting();

    void remove_card(uint32_t id, int32_t type);

    bool try_get_edit_type_id(const std::string& op_type, EDIT_TYPE_ID& out_edit_type_id)
    {
        if(JSON_KEY_OP_TYPE_UPDATE == op_type)
        {
            out_edit_type_id = ET_UPDATE;
        }
        else if(JSON_KEY_OP_TYPE_DELETE == op_type)
        {
            out_edit_type_id = ET_DELETE;
        }
        else if(JSON_KEY_OP_TYPE_INSERT == op_type)
        {
            out_edit_type_id = ET_INSERT;
        }
        else
        {
            return false;
        }

        return true;
    }

    bool is_monkeycar_area(int area_id)
    {
        char sql[1024] = {'\0'};
        sprintf(sql,"select monkeycar_areaid from dat_monkeycar_base_info where monkeycar_areaid=%d;", area_id);
        std::string Error;
        YADB::CDBResultSet DBRes;
        sDBConnPool.Query(sql,DBRes,Error);
        int64_t nCount = DBRes.GetRecordCount( Error );

        log_info("monkeycar_area区域查询：nCount=%d,sql=%s", nCount, sql);
        std_debug("monkeycar_area区域查询：nCount=%d,sql=%s", nCount, sql);

        return  (nCount > 0);
    }

    ///删除区域所有报警信息
    static void delete_area_event(std::shared_ptr<area> area_ptr)
    {
        for(int i=0; i < CARD_EVENT_COUNT_MAX; i++ )
        {
            auto ev_ptr = event_list::instance()->get_event_area(area_ptr->id(), static_cast<EVENT_TYPE>(i));
            if(ev_ptr && !ev_ptr->is_end()) //
            {
                event_list::copy_event(area_ptr, ev_ptr);

                ev_ptr->m_status = ES_END;

                event_list::save_event(ev_ptr);
            }
        }
    }

    void delete_antenna(int id)
    {
        int sitid=-8;
        auto map = sit_list::instance()->m_map;
        for(auto&it:map)
        {
            auto sit_ptr = it.second;
            if(sit_ptr->m_ant[0].m_id == id)
            {
                sitid=sit_ptr->m_id;
                sit_ptr->delete_antenna(0);
                break;
            }
            if(sit_ptr->m_ant[1].m_id == id)
            {
                sitid=sit_ptr->m_id;
                sit_ptr->delete_antenna(1);
                break;
            }
        }

        if(sitid==-8)
        {
            std_debug("天线已经删除了，在分站列表中找不到该天线：天线id=%d", id);
            log_info("天线已经删除了，在分站列表中找不到该天线：天线id=%d", id);
        }
        else
        {
            std_debug("删除天线成功：天线id=%d，分站id=%d", id, sitid);
            log_info("删除天线成功：天线id=%d，分站id=%d", id, sitid);
        }
    }

    bool is_cur_map(int id)
    {
        return cur_map_id()==id;
    }

    int cur_map_id()
    {
        auto tmp = area_list::instance()->m_map;
        for(auto&it:tmp)
        {
            if(0!=it.second->m_mapid)
            {
                return it.second->m_mapid;
            }
        }

        return -1;
    }

    void update_map_info(int id)
    {
        std::string sql = "SELECT scale FROM dat_map WHERE map_id=";

        sql.append(std::to_string(id));
        sql.append(";");

        std_debug("修改地图 sql=%s", sql.c_str());
        log_info("修改地图 sql=%s", sql.c_str());

        std::string Error;
        YADB::CDBResultSet DBRes;
        sDBConnPool.Query(sql.c_str(),DBRes,Error);
        int nCount = DBRes.GetRecordCount( Error );
        if (nCount < 1)
        {
            log_error("修改地图失败，数据库中找不到: map_id=%d", id);
            return ;
        }

        double scale  = 0;
        DBRes.GetField( "scale",scale, Error );

        auto area = area_list::instance()->m_map;
        for(auto& it : area)
        {
            it.second->m_scale = scale;
        }

        auto site = sit_list::instance()->m_map;
        for(auto& it : site)
        {
            it.second->m_scale = scale;
        }
    }

    ///收到web的删除车卡命令  保存考勤 入库  清理卡
    void clear_card_vehicle(std::shared_ptr<card_location_base> card_ptr);
};

#endif // MODULE_META_DATE_CHANGED_H
